#pragma once

#include "config.h"
#include "WatchyDisplay.h"
#include "WatchyRTC.h"
#include <WiFi.h>
#include <WiFiUdp.h>
#include <NTPClient.h>
#include "bma.h"

class Watchy
{
public:
  Watchy();
  void Init();
  void DeepSleep();
  void VibeMotor(uint8_t intervalMs = 100, uint8_t length = 20);
  float GetBatteryVoltage();
  uint64_t GetSteps();
  void ResetSteps();
  void ConnectWiFi();
  void SyncNTPTime();
  void DisconnectWiFi();
  void ShowWatchFace(bool partialRefresh = false);
  
  void ClearBusyCallback();
  void SetBusyCallback();

  // Called after hardware is setup
  virtual void Setup() = 0;

  virtual void HandleButtonPress(uint64_t buttonMask) = 0;
  virtual void HandleDoubleTap() {}
  virtual void HandleTilt() {}
  virtual void DrawWatchFace(bool partialRefresh = false) = 0;


protected:
  static void DisplayBusyCallback(const void *);

  static WatchyDisplayBase m_displayBase;
  static WatchyDisplay m_display;
  static WatchyRTC m_RTC;

  static RTC_DATA_ATTR BMA423 m_sensor;

private:
  void BmaConfig();
  static uint16_t ReadRegister(uint8_t address, uint8_t reg, uint8_t *data, uint16_t len);
  static uint16_t WriteRegister(uint8_t address, uint8_t reg, uint8_t *data, uint16_t len);
};